/**********
 * Polprog 2018
 * 3 clause BSD licence
 * http://polprog.net
 */


#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdint.h>
#include <stdlib.h>

#define DELIMETER 0x0D
#define BUFSIZE (8)

//we need a place to keep the incoming data and an index to access it
volatile uint8_t buffer[BUFSIZE];
volatile uint8_t idx;

//flag indicating that a complete number was received
volatile uint8_t process_flag = 0;

void uart_init();
void gpio_init();
void uart_print(volatile uint8_t*);

/*
 * Interrupt subroutine for UART receive interrupt.
 */
ISR(USART_RXC_vect){
  //store our character for further processing
  uint8_t c = UDR;
  if(process_flag != 0) return; //do nothing if last data was not processed yet

  //check if the character is our message delimeter
  if(c == DELIMETER){
    buffer[idx] = 0; //null terminate our string
    process_flag = 1; //flag that we need to process the data
    idx = 0; //restore index to zero for next message
    PORTB ^= 1; //debug
  }else if(idx < BUFSIZE) { //otherwise if there is space in the buffer store it
    buffer[idx] = c; //write the character to the buffer
    //UDR = c; //echo the character back
    idx++; //increment index
  }
}



void uart_init(){
  // set baudrate to 9600 (U2X enabled, 1MHz clock)
  UBRRH = 0;
  UBRRL = 12;
  // enable 2x speed mode 
  UCSRA |= (1<<U2X);
  // Enable receiver, transmitter, and receive complete interrupt
  UCSRB = (1<<RXEN)|(1<<TXEN)|(1<<RXCIE);
  // Set frame format: 8data, 2 stop bits
  UCSRC = (1<<URSEL)|(1<<USBS)|(3<<UCSZ0);
}

void gpio_init(){
  //LED on PB0
  DDRB |= (1<<0);
}

void uart_print(volatile uint8_t* message){
  while(*message != 0){
    while(!(UCSRA & (1<<UDRE)));
    UDR = *message;
    message++;
  }
}

int main(){
  //initialize GPIO
  gpio_init();
  //initialize UART
  uart_init();
  //initialize buffer
  for(idx = 0; idx < BUFSIZE; idx++){
    buffer[idx] = 0;
  }
  idx = 0;


  //enable interrupts
  sei();


  //define a variable to store the number (16 bit) and an output buffer
  uint16_t number;
  uint8_t output[BUFSIZE];
  while(1){
    while(process_flag == 0); //wait for process flag to be 1
    
    number = atol(buffer); //convert from input buffer to a number
    number *= 2; //perform operation on the number
    ltoa(number, output, 10); //convert back
    
    uart_print(output); //output the result
    uart_print("\n");
    process_flag = 0;
  }

}
